% DISPRELDUST Main program for finding the dispersion relation for waves
% in dusty plasmas, or other plasmas approximating one ion population by
% omega_d^2/omega^2 in Eq. (6) of Gunell and Skiff, Phys. Plasmas,
% vol. 8, 3550-3557 (2001).
%
% [vph, UHPpole, lim, omega, f, k, v] = DISPRELDUST('inputfile',ompd2,lde2,k,v)
%
% inputfile: m-file which defines the poles and residues of the distribution
% ompd2:     dust plasma frequency squared
% lde2:      electron Debye length squared
% k:         wave number vector for which the solution is computed
% v:         velocity vector for which f is computed
%
% vph:     root paths in the complex phase velocity plane
% UHPpole: the poles of the distribution function in the upper half plane
% lim:     limiting solutions of vph for k=0
% omega:   complex angular frequency, Im{omega}>0 for stable solutions
% f:       distribution function

function [vph,UHPpole,lim,omega,f,k,v]=dispreldust(inputfile,ompd2,lde2,k,v)


% Defaults
if nargin<5
  v=[-3:0.005:3];
  if nargin<4
    k=[2.^(-6:0.1:8.3)];
  end
end

% Get the poles and residues
run(inputfile)

%%%% Limiting solution, k=0
P = polyion(UHPpole,UHPresid,0,ompd2,lde2); % Finds disp.rel. polynomial

lim=roots(P);                              % Solution to Eq.

sol_old=[1e6 -1e6 lim.'];                  % Initial value for the sort


omega=[]; vph=[];
for ii=1:length(k)
  C=k(ii)^2;
  P = polyion(UHPpole,UHPresid,C,ompd2,lde2);

% Issue a warning if the moduli of some coefficients are much larger than
% others
  if min(abs(P))/max(abs(P))<eps
    warning(['min(abs(P))/max(abs(P)) = ' ...
             num2str(min(abs(P))/max(abs(P)))])
  end

  solution=roots(P);

  sol_sorted = refsort(sol_old, solution); %This is where the equation is solved
  sol_old = sol_sorted;    
         
  vph=[vph sol_sorted];
  omega = [omega k(ii)*sol_sorted];     % Sorts the roots into individual modes
end

% Distribution function
f=sum(ALLresid.'*ones(size(v))./ ...
       (ones(size(ALLpole)).'*v - ALLpole.'*ones(size(v))));
f=real(f);
